"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = createGenerator;

var _utils = require("@elodin/utils");

var _cssInJsUtils = require("css-in-js-utils");

var _getReset = require("./getReset");

var _module = require("module");

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var defaultConfig = {
  devMode: false,
  generateResetClassName: function generateResetClassName(type) {
    return '_elo_' + type;
  },
  generateCSSFileName: function generateCSSFileName(moduleName) {
    return moduleName + '.elo';
  },
  generateJSFileName: function generateJSFileName(moduleName) {
    return moduleName + '.elo';
  }
};

function createGenerator() {
  var customConfig = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var config = _objectSpread({}, defaultConfig, {}, customConfig);

  var adapter = config.adapter,
      generateResetClassName = config.generateResetClassName,
      rootNode = config.rootNode;

  if (!adapter) {
    throw new Error('An adapter needs to passed in order to generate code. See @elodin/generator-css-in-js/lib/adapters for more information.');
  }

  var cssReset = (0, _getReset.baseReset)(config.generateResetClassName);

  if (config.rootNode) {
    cssReset += (0, _getReset.rootReset)(config.rootNode);
  }

  function generate(ast) {
    var _objectSpread2;

    var path = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
    var fileName = path.split('/').pop();
    var relativeRootPath = '../'.repeat(path.split('/').length - 1);
    config.relativeRootPath = relativeRootPath || './';
    var css = generateCSS(ast, config);
    var js = generateJS(ast, config, adapter);
    var root = generateRoot(ast, config);
    return _objectSpread((_objectSpread2 = {}, _defineProperty(_objectSpread2, relativeRootPath + config.generateCSSFileName('_reset') + '.css', cssReset), _defineProperty(_objectSpread2, fileName + '.js', root), _objectSpread2), css, {}, js);
  }

  generate.filePattern = [config.generateJSFileName('*') + '.js', config.generateCSSFileName('*') + '.css'];
  generate.ignorePattern = ['node_modules'];
  return generate;
}

function generateRoot(ast, _ref) {
  var relativeRootPath = _ref.relativeRootPath,
      generateCSSFileName = _ref.generateCSSFileName,
      generateJSFileName = _ref.generateJSFileName;
  // TODO: include fragments
  var styles = ast.body.filter(function (node) {
    return node.type === 'Style';
  });
  var imports = styles.map(function (module) {
    return 'import { ' + module.name + " } from './" + generateJSFileName(module.name) + ".js'";
  }).join('\n');
  return "import '" + relativeRootPath + generateCSSFileName('_reset') + ".css'\n" + imports + '\n\n' + 'export {\n  ' + styles.map(function (module) {
    return module.name;
  }).join(',\n  ') + '\n}';
}

function generateCSS(ast, _ref2) {
  var devMode = _ref2.devMode,
      generateCSSFileName = _ref2.generateCSSFileName;
  // TODO: include fragments
  var styles = ast.body.filter(function (node) {
    return node.type === 'Style';
  });
  var variants = ast.body.filter(function (node) {
    return node.type === 'Variant';
  });
  return styles.reduce(function (files, module) {
    var usedVariants = (0, _utils.getVariantsFromAST)(module);
    var variantMap = variants.reduce(function (flatVariants, variant) {
      if (usedVariants[variant.name]) {
        flatVariants[variant.name] = variant.body.map(function (variation) {
          return variation.value;
        });
      }

      return flatVariants;
    }, {});
    var classes = (0, _utils.generateCSSClasses)(module.body, variantMap, devMode);
    files[generateCSSFileName(module.name) + '.css'] = classes.filter(function (selector) {
      return selector.declarations.length > 0;
    }).map(function (selector) {
      var css = (0, _utils.stringifyCSSRule)(selector.declarations, (0, _utils.getModuleName)(module, devMode) + selector.modifier + selector.pseudo, selector.media ? '  ' : '');

      if (selector.media) {
        return '@media ' + selector.media + ' {\n' + css + '\n}';
      }

      return css;
    }).join('\n\n');
    return files;
  }, {});
}

function generateJS(ast, _ref3, adapter) {
  var devMode = _ref3.devMode,
      generateResetClassName = _ref3.generateResetClassName,
      generateCSSFileName = _ref3.generateCSSFileName,
      generateJSFileName = _ref3.generateJSFileName;
  // TODO: include fragments
  var styles = ast.body.filter(function (node) {
    return node.type === 'Style';
  });
  var variants = ast.body.filter(function (node) {
    return node.type === 'Variant';
  });
  return styles.reduce(function (files, module) {
    var usedVariants = (0, _utils.getVariantsFromAST)(module);
    var variantMap = variants.reduce(function (flatVariants, variant) {
      if (usedVariants[variant.name]) {
        flatVariants[variant.name] = variant.body.map(function (variation) {
          return variation.value;
        });
      }

      return flatVariants;
    }, {});
    var variables = (0, _utils.getVariablesFromAST)(module);
    var style = generateStyle(module.body);
    var classNameMap = generateClassNameMap(module.body, variantMap, devMode);
    var variantStyleMap = generateVariantStyleMap(module.body, variants);
    files[generateJSFileName(module.name) + '.js'] = adapter({
      style: style,
      variantStyleMap: variantStyleMap,
      variables: variables,
      moduleName: module.name,
      cssFileName: generateCSSFileName(module.name),
      classNameMap: classNameMap,
      resetClassName: generateResetClassName(module.format),
      className: (0, _utils.getModuleName)(module, devMode),
      variants: variantMap
    });
    return files;
  }, {});
}

function generateClassNameMap(nodes, variantMap, devMode) {
  var classes = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
  var variations = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {};
  var modifier = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : [];
  var nesting = nodes.filter(function (node) {
    return node.type !== 'Declaration';
  });
  var variantOrder = Object.keys(variantMap); // ensure the variant modifier order is always deterministic

  classes[modifier.sort(function (a, b) {
    return variantOrder.indexOf(a[0]) > variantOrder.indexOf(b[0]) ? 1 : -1;
  }).map(function (_ref4) {
    var _ref5 = _slicedToArray(_ref4, 2),
        name = _ref5[0],
        value = _ref5[1];

    return devMode ? '__' + name + '-' + value : '_' + variantOrder.indexOf(name) + '-' + variantMap[name].indexOf(value);
  }).join('')] = variations;
  nesting.forEach(function (nest) {
    if (nest.property.type === 'Identifier') {
      var variant = variantMap[nest.property.value];

      if (variant) {
        if (nest.value.type === 'Identifier') {
          var variation = variant.indexOf(nest.value.value) !== -1;

          if (variation) {
            generateClassNameMap(nest.body, variantMap, devMode, classes, _objectSpread({}, variations, _defineProperty({}, nest.property.value, nest.value.value)), [].concat(_toConsumableArray(modifier), [[nest.property.value, nest.value.value]]));
          }
        }
      } else {// TODO: throw
      }
    }
  });
  return classes;
}

function generateVariantStyleMap(nodes, variants) {
  var styles = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var style = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : [];
  var modifier = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : [];
  var nesting = nodes.filter(function (node) {
    return node.type !== 'Declaration';
  });
  var variantOrder = variants.map(function (variant) {
    return variant.name;
  });

  if (style.length > 0) {
    styles[modifier.sort(function (a, b) {
      return variantOrder.indexOf(a[0]) > variantOrder.indexOf(b[0]) ? 1 : -1;
    }).map(function (_ref6) {
      var _ref7 = _slicedToArray(_ref6, 2),
          name = _ref7[0],
          value = _ref7[1];

      return '__' + name + '-' + value;
    }).join('')] = style;
  }

  nesting.forEach(function (nest) {
    if (nest.property.type === 'Identifier') {
      var variant = variants.find(function (variant) {
        return variant.name === nest.property.value;
      });

      if (variant) {
        if (nest.value.type === 'Identifier') {
          var variation = variant.body.find(function (variant) {
            return variant.value === nest.value.value;
          });

          if (variation) {
            generateVariantStyleMap(nest.body, variants, styles, generateStyle(nest.body), [].concat(_toConsumableArray(modifier), [[variant.name, variation.value]]));
          }
        }
      } else {// TODO: throw
      }
    }
  });
  return styles;
}

function generateStyle(nodes) {
  var base = nodes.filter(function (node) {
    return node.type === 'Declaration';
  });
  var nestings = nodes.filter(function (node) {
    return node.type !== 'Declaration';
  });
  var declarations = base.filter(function (decl) {
    return decl.dynamic;
  }).map(function (decl) {
    return {
      property: decl.property,
      value: generateValue(decl.value, decl.property === 'opacity')
    };
  });
  var nests = nestings.map(function (nest) {
    if (nest.property.type === 'Variable' && nest.property.environment) {
      var pseudoClass = (0, _utils.isPseudoClass)(nest.property.value);
      var pseudoElement = (0, _utils.isPseudoElement)(nest.property.value);
      var mediaQuery = (0, _utils.isMediaQuery)(nest.property.value);

      if ((pseudoClass || pseudoElement) && nest["boolean"]) {
        return {
          property: (pseudoElement ? '::' : ':') + nest.property.value,
          value: generateStyle(nest.body)
        };
      }

      if (mediaQuery) {
        return {
          property: '@media ' + (0, _utils.generateCSSMediaQueryFromNode)(nest["boolean"] ? undefined : nest.value.value, nest.property.value, nest.operator),
          value: generateStyle(nest.body)
        };
      }
    }
  }).filter(function (nesting) {
    return nesting && nesting.value.length > 0;
  });
  return [].concat(_toConsumableArray(declarations), _toConsumableArray(nests));
}

function wrapInString(value) {
  return "'" + value + "'";
}

function wrapInParens(value) {
  return '(' + value + ')';
}

var inlineFns = {
  add: ' + ',
  sub: ' - ',
  mul: ' * ',
  div: ' / ',
  percentage: true
};
var stringFns = {
  rgb: true,
  rgba: true,
  hsl: true,
  hsla: true
};

function generateFunction(node) {
  var floatingPercentage = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

  if (stringFns[node.callee]) {
    return wrapInString(node.callee + '(' + node.params.map(function (param) {
      if (param.type === 'Variable' || param.type === 'FunctionExpression' && inlineFns[param.callee]) {
        return "' + " + wrapInParens(generateValue(param, true)) + " + '";
      }

      return generateValue(param, true);
    }).join(', ') + ')');
  }

  if (node.callee === 'percentage') {
    if (floatingPercentage) {
      return wrapInParens(generateValue(node.params[0], floatingPercentage) + ' / 100');
    }

    return '(' + generateValue(node.params[0], floatingPercentage) + ") + '%'";
  }

  if (node.callee === 'raw') {
    return generateValue(node.params[0], floatingPercentage);
  }

  if (inlineFns[node.callee]) {
    return wrapInParens(node.params.map(function (value) {
      return generateValue(value, floatingPercentage);
    }).join(inlineFns[node.callee]));
  }
}

function generateValue(node) {
  var floatingPercentage = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

  if (node.type === 'FunctionExpression') {
    return generateFunction(node, floatingPercentage);
  }

  if (node.type === 'Integer') {
    return (node.negative ? '-' : '') + node.value;
  }

  if (node.type === 'Float') {
    return (node.negative ? '-' : '') + node.integer + '.' + node.fractional;
  }

  if (node.type === 'Identifier') {
    return (0, _cssInJsUtils.hyphenateProperty)(node.value);
  }

  if (node.type === 'Variable') {
    return 'props.' + node.value;
  }

  return node.value;
}